/* signatures of BDDCML interface functions */
/* meaning of the variables is described in src/bddcml_fortran_interface.f90 */
#ifndef bddcml_h
#define bddcml_h

// macro for calling Fortran routines from C - changing the name as Fortran compiler does it
#ifndef F_SYMBOL

# if defined(UPPER) 
#  define F_SYMBOL(lower_case,upper_case) upper_case
# elif defined(Add_)
#  define F_SYMBOL(lower_case,upper_case) lower_case##_
# elif defined(Add__)
#  define F_SYMBOL(lower_case,upper_case) lower_case##__
# else
#  define F_SYMBOL(lower_case,upper_case) lower_case
# endif

# endif

#define bddcml_init F_SYMBOL(bddcml_init, BDDCML_INIT)
void bddcml_init( int *nl, int *nsublev, int *lnsublev, int *nsub_loc_1, int *comm_init, int *verbose_level, int *numbase );

#define bddcml_upload_global_data F_SYMBOL(bddcml_upload_global_data, BDDCML_UPLOAD_GLOBAL_DATA)
void bddcml_upload_global_data( int *nelem, int *nnod, int *ndof, int *ndim, int *meshdim, 
                                int *inet, int *linet, int *nnet, int *lnnet, int *nndf, int *lnndf, 
                                double *xyz, int *lxyz1, int *lxyz2,
                                int *ifix, int *lifix, double *fixv, int *lfixv, double *rhs, int *lrhs, double *sol, int *lsol, int *idelm, 
                                int *neighbouring, int *load_division_int );

#define bddcml_upload_subdomain_data F_SYMBOL(bddcml_upload_subdomain_data, BDDCML_UPLOAD_SUBDOMAIN_DATA)
void bddcml_upload_subdomain_data( int *nelem, int *nnod, int *ndof, int *ndim, int *meshdim,
                                   int *isub, int *nelems, int *nnods, int *ndofs, 
                                   int *inet, int *linet, int *nnet, int *lnnet, int *nndf, int *lnndf, 
                                   int *isngn, int *lisngn, int *isvgvn, int *lisvgvn, int *isegn, int *lisegn, 
                                   double *xyz, int *lxyz1, int *lxyz2, 
                                   int *ifix, int *lifix, double *fixv, int *lfixv, 
                                   double *rhs, int *lrhs, int *is_rhs_complete, 
                                   double *sol, int *lsol, 
                                   int *matrixtype, int *i_sparse, int *j_sparse, double *a_sparse, int *la, int *is_assembled_int );

#define bddcml_setup_preconditioner F_SYMBOL(bddcml_setup_preconditioner, BDDCML_SETUP_PRECONDITIONER)
void bddcml_setup_preconditioner( int *matrixtype, int *use_defaults_int,
                                  int *parallel_division_int, int *use_arithmetic_int, int *use_adaptive_int );

#define bddcml_solve F_SYMBOL(bddcml_solve, BDDCML_SOLVE)
void bddcml_solve( int *comm_all, int *method, double *tol, int *maxit, int *ndecrmax, 
                   int *num_iter, int *converged_reason, double *condition_number);

#define bddcml_download_local_solution F_SYMBOL(bddcml_download_local_solution, BDDCML_DOWNLOAD_LOCAL_SOLUTION)
void bddcml_download_local_solution( int *isub, double *sols, int *lsols );

#define bddcml_download_global_solution F_SYMBOL(bddcml_download_global_solution, BDDCML_DOWNLOAD_GLOBAL_SOLUTION)
void bddcml_download_global_solution( double *sol, int *lsol );

#define bddcml_dotprod_subdomain F_SYMBOL(bddcml_dotprod_subdomain, BDDCML_DOTPROD_SUBDOMAIN)
void bddcml_dotprod_subdomain( int *isub, double *vec1, int *lvec1, double *vec2, int *lvec2, double *dotprod );

#define bddcml_finalize F_SYMBOL(bddcml_finalize, BDDCML_FINALIZE)
void bddcml_finalize( );

#endif
